@@ -52,14 +52,14 @@ struct notcurses* notcurses_init(const notcurses_options*, FILE*);
 void notcurses_version_components(int* major, int* minor, int* patch, int* tweak);
 int notcurses_lex_margins(const char* op, notcurses_options* opts);
 int notcurses_stop(struct notcurses*);
-int notcurses_render(struct notcurses*);
+int notcurses_render(struct notcurses* nc);
+int notcurses_render_to_buffer(struct notcurses* nc, char** buf, size_t* buflen);
 int notcurses_render_to_file(struct notcurses* nc, FILE* fp);
 struct ncplane* notcurses_stdplane(struct notcurses*);
 const struct ncplane* notcurses_stdplane_const(const struct notcurses* nc);
 void ncplane_set_channels(struct ncplane* n, uint64_t channels);
 uint64_t ncplane_set_fchannel(struct ncplane* n, uint32_t channel);
 uint64_t ncplane_set_bchannel(struct ncplane* n, uint32_t channel);
-void ncplane_set_attr(struct ncplane* n, unsigned stylebits);
 int ncplane_set_base_cell(struct ncplane* ncp, const cell* c);
 int ncplane_set_base(struct ncplane* ncp, const char* egc, uint32_t styles, uint64_t channels);
 int ncplane_base(struct ncplane* ncp, cell* c);
@@ -73,12 +73,20 @@ typedef enum {
   NCALIGN_CENTER,
   NCALIGN_RIGHT,
 } ncalign_e;
-struct ncplane* ncplane_new(struct notcurses* nc, int rows, int cols, int yoff, int xoff, void* opaque);
-struct ncplane* ncplane_new_named(struct notcurses* nc, int rows, int cols, int yoff, int xoff, void* opaque, const char* name);
-struct ncplane* ncplane_bound(struct ncplane* n, int rows, int cols, int yoff, int xoff, void* opaque);
-struct ncplane* ncplane_bound_named(struct ncplane* n, int rows, int cols, int yoff, int xoff, void* opaque, const char* name);
-struct ncplane* ncplane_aligned(struct ncplane* n, int rows, int cols, int yoff, ncalign_e align, void* opaque);
-struct ncplane* ncplane_aligned_named(struct ncplane* n, int rows, int cols, int yoff, ncalign_e align, void* opaque, const char* name);
+typedef struct ncplane_options {
+  int y;            // vertical placement relative to parent plane
+  union {
+    int x;
+    ncalign_e align;
+  } horiz;          // horizontal placement relative to parent plane
+  int rows;         // number of rows, must be positive
+  int cols;         // number of columns, must be positive
+  void* userptr;    // user curry, may be NULL
+  const char* name; // name (used only for debugging), may be NULL
+  int (*resizecb)(struct ncplane*); // callback when parent is resized
+  uint64_t flags;   // closure over NCPLANE_OPTION_*
+} ncplane_options;
+struct ncplane* ncplane_create(struct ncplane* n, const ncplane_options* nopts);
 unsigned notcurses_supported_styles(const struct notcurses* nc);
 unsigned notcurses_palette_size(const struct notcurses* nc);
 bool notcurses_cantruecolor(const struct notcurses* nc);
@@ -130,23 +138,22 @@ void* ncplane_userptr(struct ncplane* n);
 int ncplane_resize(struct ncplane* n, int keepy, int keepx, int keepleny,
                        int keeplenx, int yoff, int xoff, int ylen, int xlen);
 uint64_t ncplane_channels(const struct ncplane* n);
-uint16_t ncplane_attr(const struct ncplane* n);
-unsigned ncplane_styles(const struct ncplane* n);
-int ncplane_set_fg_rgb(struct ncplane* n, int r, int g, int b);
-int ncplane_set_bg_rgb(struct ncplane* n, int r, int g, int b);
-void ncplane_set_fg_rgb_clipped(struct ncplane* n, int r, int g, int b);
-void ncplane_set_bg_rgb_clipped(struct ncplane* n, int r, int g, int b);
-int ncplane_set_fg(struct ncplane* n, unsigned channel);
-int ncplane_set_bg(struct ncplane* n, unsigned channel);
+int ncplane_set_fg_rgb8(struct ncplane* n, int r, int g, int b);
+int ncplane_set_bg_rgb8(struct ncplane* n, int r, int g, int b);
+void ncplane_set_fg_rgb8_clipped(struct ncplane* n, int r, int g, int b);
+void ncplane_set_bg_rgb8_clipped(struct ncplane* n, int r, int g, int b);
+int ncplane_set_fg_rgb(struct ncplane* n, unsigned channel);
+int ncplane_set_bg_rgb(struct ncplane* n, unsigned channel);
 void ncplane_set_fg_default(struct ncplane* n);
 void ncplane_set_bg_default(struct ncplane* n);
 int ncplane_set_fg_alpha(struct ncplane* n, unsigned alpha);
 int ncplane_set_bg_alpha(struct ncplane* n, unsigned alpha);
 int ncplane_set_fg_palindex(struct ncplane* n, int idx);
 int ncplane_set_bg_palindex(struct ncplane* n, int idx);
-void ncplane_styles_set(struct ncplane* n, unsigned stylebits);
-void ncplane_styles_on(struct ncplane* n, unsigned stylebits);
-void ncplane_styles_off(struct ncplane* n, unsigned stylebits);
+unsigned ncplane_styles(const struct ncplane* n);
+void ncplane_set_styles(struct ncplane* n, unsigned stylebits);
+void ncplane_on_styles(struct ncplane* n, unsigned stylebits);
+void ncplane_off_styles(struct ncplane* n, unsigned stylebits);
 typedef struct ncstats {
   uint64_t renders;          // number of successful notcurses_render() runs
   uint64_t failed_renders;   // number of aborted renders, should be 0
@@ -166,8 +173,9 @@ typedef struct ncstats {
   uint64_t defaultelisions;  // default color was emitted
   uint64_t defaultemissions; // default color was elided
 } ncstats;
+ncstats* notcurses_stats_alloc(struct notcurses* nc);
 void notcurses_stats(struct notcurses* nc, ncstats* stats);
-void notcurses_reset_stats(struct notcurses* nc, ncstats* stats);
+void notcurses_stats_reset(struct notcurses* nc, ncstats* stats);
 int ncplane_hline_interp(struct ncplane* n, const cell* c, int len, uint64_t c1, uint64_t c2);
 int ncplane_vline_interp(struct ncplane* n, const cell* c, int len, uint64_t c1, uint64_t c2);
 int ncplane_box(struct ncplane* n, const cell* ul, const cell* ur, const cell* ll, const cell* lr, const cell* hline, const cell* vline, int ystop, int xstop, unsigned ctlword);
@@ -177,7 +185,8 @@ int ncplane_fadein(struct ncplane* n, const struct timespec* ts, fadecb fader, v
 int ncplane_pulse(struct ncplane* n, const struct timespec* ts, fadecb fader, void* curry);
 int ncplane_putegc_yx(struct ncplane* n, int y, int x, const char* gclust, int* sbytes);
 int ncplane_putstr_aligned(struct ncplane* n, int y, ncalign_e align, const char* s);
-int ncplane_putstr_stainable(struct ncplane* n, const char* s);
+int ncplane_putstr_stained(struct ncplane* n, const char* s);
+int ncplane_putwstr_stained(struct ncplane* n, const wchar_t* gclustarr);
 struct ncplane* ncplane_dup(const struct ncplane* n, void* opaque);
 int cell_load(struct ncplane* n, cell* c, const char* gcluster);
 int cell_duplicate(struct ncplane* n, cell* targ, const cell* c);
@@ -246,9 +255,9 @@ typedef struct ncselector_options {
   uint64_t titlechannels;// title channels
   uint64_t footchannels; // secondary and footer channels
   uint64_t boxchannels;  // border channels
-  uint64_t bgchannels;   // background channels, used only in body
+  uint64_t flags;        // bitmap over NCSELECTOR_OPTIONS_*
 } ncselector_options;
-struct ncselector* ncselector_create(struct ncplane* n, int y, int x, const ncselector_options* opts);
+struct ncselector* ncselector_create(struct ncplane* n, const ncselector_options* opts);
 int ncselector_additem(struct ncselector* n, const struct ncselector_item* item);
 int ncselector_delitem(struct ncselector* n, const char* item);
 const char* ncselector_selected(const struct ncselector* n);
@@ -275,9 +284,9 @@ typedef struct ncmultiselector_options {
   uint64_t titlechannels;// title channels
   uint64_t footchannels; // secondary and footer channels
   uint64_t boxchannels;  // border channels
-  uint64_t bgchannels;   // background channels, used only in body
+  uint64_t flags;        // bitmap over NCSELECTOR_OPTIONS_*
 } ncmultiselector_options;
-struct ncmultiselector* ncmultiselector_create(struct ncplane* n, int y, int x, const ncmultiselector_options* opts);
+struct ncmultiselector* ncmultiselector_create(struct ncplane* n, const ncmultiselector_options* opts);
 int ncmultiselector_selected(struct ncmultiselector* n, bool* selected, unsigned count);
 struct ncplane* ncmultiselector_plane(struct ncmultiselector* n);
 bool ncmultiselector_offer_input(struct ncmultiselector* n, const struct ncinput* nc);
@@ -316,7 +325,6 @@ typedef struct ncreel_options {
   unsigned tabletmask;
   uint64_t tabletchan;
   uint64_t focusedchan;
-  uint64_t bgchannel;
   unsigned flags;      // bitfield over NCREEL_OPTION_*
 } ncreel_options;
 struct ncreel* ncreel_create(struct ncplane* nc, const ncreel_options* popts);
@@ -329,16 +337,16 @@ int ncreel_redraw(struct ncreel* pr);
 struct nctablet* ncreel_focused(struct ncreel* pr);
 struct nctablet* ncreel_next(struct ncreel* pr);
 struct nctablet* ncreel_prev(struct ncreel* pr);
-int ncreel_destroy(struct ncreel* pr);
+void ncreel_destroy(struct ncreel* pr);
 void* nctablet_userptr(struct nctablet* t);
-struct ncplane* nctablet_ncplane(struct nctablet* t);
+struct ncplane* nctablet_plane(struct nctablet* t);
 int ncplane_polyfill_yx(struct ncplane* n, int y, int x, const cell* c);
 int ncplane_gradient(struct ncplane* n, const char* egc, uint32_t styles, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, int ystop, int xstop);
 int ncplane_highgradient(struct ncplane* n, uint32_t ul, uint32_t ur, uint32_t ll, uint32_t lr, int ystop, int xstop);
 int ncplane_highgradient_sized(struct ncplane* n, uint32_t ul, uint32_t ur, uint32_t ll, uint32_t lr, int ylen, int xlen);
-int ncplane_putsimple_stainable(struct ncplane* n, char c);
-int ncplane_putegc_stainable(struct ncplane* n, const char* gclust, int* sbytes);
-int ncplane_putwegc_stainable(struct ncplane* n, const wchar_t* gclust, int* sbytes);
+int ncplane_putchar_stained(struct ncplane* n, char c);
+int ncplane_putegc_stained(struct ncplane* n, const char* gclust, int* sbytes);
+int ncplane_putwegc_stained(struct ncplane* n, const wchar_t* gclust, int* sbytes);
 int ncplane_format(struct ncplane* n, int ystop, int xstop, uint32_t styles);
 int ncplane_stain(struct ncplane* n, int ystop, int xstop, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr);
 int ncplane_rotate_cw(struct ncplane* n);
@@ -353,6 +361,7 @@ typedef struct ncplot_options {
   uint64_t rangex;
   unsigned flags;
   const char* title;
+  uint64_t flags;        // bitmap over NCPLOT_OPTIONS_*
 } ncplot_options;
 struct ncuplot* ncuplot_create(struct ncplane* n, const ncplot_options* opts, uint64_t miny, uint64_t maxy);
 struct ncdplot* ncdplot_create(struct ncplane* n, const ncplot_options* opts, double miny, double maxy);
@@ -370,6 +379,7 @@ bool ncplane_set_scrolling(struct ncplane* n, bool scrollp);
 typedef struct ncfdplane_options {
   void* curry; // parameter provided to callbacks
   bool follow; // keep reading after hitting end? (think tail -f)
+  uint64_t flags;        // bitmap over NCPLOT_OPTIONS_*
 } ncfdplane_options;
 typedef int(ncfdplane_callback)(struct ncfdplane* n, const void* buf, size_t s, void* curry);
 typedef int(ncfdplane_done_cb)(struct ncfdplane* n, int fderrno, void* curry);
@@ -379,6 +389,7 @@ int ncfdplane_destroy(struct ncfdplane* n);
 typedef struct ncsubproc_options {
   void* curry;
   uint64_t restart_period;  // restart this many seconds after an exit (watch)
+  uint64_t flags;        // bitmap over NCPLOT_OPTIONS_*
 } ncsubproc_options;
 struct ncsubproc* ncsubproc_createv(struct ncplane* n, const ncsubproc_options* opts, const char* bin, char* const arg[], ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn);
 struct ncsubproc* ncsubproc_createvp(struct ncplane* n, const ncsubproc_options* opts, const char* bin, char* const arg[], ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn);
@@ -387,15 +398,10 @@ struct ncplane* ncsubproc_plane(struct ncsubproc* n);
 int ncsubproc_destroy(struct ncsubproc* n);
 typedef struct ncreader_options {
   uint64_t tchannels; // channels used for input
-  uint64_t echannels; // channels used for empty space
   uint32_t tattrword; // attributes used for input
-  uint32_t eattrword; // attributes used for empty space
-  const char* egc;    // egc used for empty space
-  int physrows;
-  int physcols;
   unsigned flags;     // bitfield over NCREADER_OPTION_*
 } ncreader_options;
-struct ncreader* ncreader_create(struct ncplane* n, int y, int x, const ncreader_options* opts);
+struct ncreader* ncreader_create(struct ncplane* n, const ncreader_options* opts);
 int ncreader_clear(struct ncreader* n);
 struct ncplane* ncreader_plane(struct ncreader* n);
 bool ncreader_offer_input(struct ncreader* n, const struct ncinput* ni);
@@ -406,12 +412,12 @@ int ncplane_putnstr_yx(struct ncplane* n, int y, int x, size_t s, const char* gc
 int ncplane_putnstr_aligned(struct ncplane* n, int y, ncalign_e align, size_t s, const char* gclustarr);
 int ncplane_qrcode(struct ncplane* n, ncblitter_e blitter, int* ymax, int* xmax, const void* data, size_t len);
 struct ncdirect* ncdirect_init(const char* termtype, FILE* fp, uint64_t flags);
-int ncdirect_bg_rgb(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
-int ncdirect_fg_rgb(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
+int ncdirect_bg_rgb8(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
+int ncdirect_fg_rgb8(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
 unsigned ncdirect_palette_size(const struct ncdirect* nc);
 int ncdirect_putstr(struct ncdirect* nc, uint64_t channels, const char* utf8);
-int ncdirect_fg(struct ncdirect* n, unsigned rgb);
-int ncdirect_bg(struct ncdirect* n, unsigned rgb);
+int ncdirect_fg_rgb(struct ncdirect* n, unsigned rgb);
+int ncdirect_bg_rgb(struct ncdirect* n, unsigned rgb);
 int ncdirect_styles_set(struct ncdirect* n, unsigned stylebits);
 int ncdirect_styles_on(struct ncdirect* n, unsigned stylebits);
 int ncdirect_styles_off(struct ncdirect* n, unsigned stylebits);
@@ -444,6 +450,7 @@ int ncreader_move_right(struct ncreader* n);
 int ncreader_move_up(struct ncreader* n);
 int ncreader_move_down(struct ncreader* n);
 int ncreader_write_egc(struct ncreader* n, const char* egc);
+int ncstrwidth(const char* text);
 """)
 
 if __name__ == "__main__":
